
/**
 * Copyright 2012 Clint Combs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import com.mongodb.casbah.Imports._
import java.net.InetAddress.getLocalHost

package org {
  package object newhart {
    import java.io.{PrintWriter,StringWriter}

    trait MongoDBObjectConverter[T] {
      def toDB(t:T):MongoDBObject
      def fromDB(obj:MongoDBObject):T
    }

    def auditParameter(p:Any) = {
      println(p)
    }

    def auditReturnValue[R](r:R):R = {
      println("return value: " + r)
      r
    }

    def auditException(e:Exception) = {
      println("Exception: " + e)
    }

    // TODO: rename to "audit"

    // 0 parameters
    def auditCall[R](f:Function0[R]):R = {
      val r:R = f()
      auditReturnValue(r)
    }

    def auditCall[R](expr:R):R = {
      try {
        auditReturnValue(expr)
      } catch {
        case e:Exception => { auditException(e); throw e }
      }
    }

    def auditData[R](name:String)(expr:R):R = {
      println("audit data: " + name + "=" + expr)
      expr
    }

    // 1 parameter
    def auditCall[T1, R](f:Function1[T1, R])(t1:T1):R = {
      auditParameter(t1)
      auditReturnValue(f(t1))
    }

    // 2 parameters
    def auditCall[T1, T2, R](f:Function2[T1, T2, R])(t1:T1, t2:T2):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditReturnValue(f(t1, t2))
    }

    // 3 parameters
    def auditCall[T1, T2, T3, R](f:Function3[T1, T2, T3, R])(t1:T1, t2:T2, t3:T3):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditReturnValue(f(t1, t2, t3))
    }

    // 4 parameters
    def auditCall[T1, T2, T3, T4, R](f:Function4[T1, T2, T3, T4, R])(t1:T1, t2:T2, t3:T3, t4:T4):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditReturnValue(f(t1, t2, t3, t4))
    }

    // 5 parameters
    def auditCall[T1, T2, T3, T4, T5, R]
    (f:Function5[T1, T2, T3, T4, T5, R])(t1:T1, t2:T2, t3:T3, t4:T4, t5:T5):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditReturnValue(f(t1, t2, t3, t4, t5))
    }

    // 6 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, R]
    (f:Function6[T1, T2, T3, T4, T5, T6, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6))
    }

    // 7 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, R]
    (f:Function7[T1, T2, T3, T4, T5, T6, T7, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7))
    }

    // 8 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, R]
    (f:Function8[T1, T2, T3, T4, T5, T6, T7, T8, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8))
    }

    // 9 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, R]
    (f:Function9[T1, T2, T3, T4, T5, T6, T7, T8, T9, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9))
    }

    // 10 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, R]
    (f:Function10[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10))
    }

    // 11 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, R]
    (f:Function11[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11))
    }

    // 12 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, R]
    (f:Function12[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12))
    }

    // 13 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, R]
    (f:Function13[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13))
    }

    // 14 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, R]
    (f:Function14[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14))
    }

    // 15 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, R]
    (f:Function15[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15))
    }

    // 16 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, R]
    (f:Function16[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15, t16:T16):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditParameter(t16)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15, t16))
    }

    // 17 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, R]
    (f:Function17[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15, t16:T16, t17:T17):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditParameter(t16)
      auditParameter(t17)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15, t16, t17))
    }

    // 18 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, R]
    (f:Function18[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15, t16:T16, t17:T17, t18:T18):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditParameter(t16)
      auditParameter(t17)
      auditParameter(t18)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15, t16, t17, t18))
    }

    // 19 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, R]
    (f:Function19[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15, t16:T16, t17:T17, t18:T18, t19:T19):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditParameter(t16)
      auditParameter(t17)
      auditParameter(t18)
      auditParameter(t19)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15, t16, t17, t18, t19))
    }

    // 20 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, T20, R]
    (f:Function20[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, T20, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15, t16:T16, t17:T17, t18:T18, t19:T19, t20:T20):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditParameter(t16)
      auditParameter(t17)
      auditParameter(t18)
      auditParameter(t19)
      auditParameter(t20)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15, t16, t17, t18, t19, t20))
    }

    // 21 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, T20, T21, R]
    (f:Function21[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, T20, T21, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15, t16:T16, t17:T17, t18:T18, t19:T19, t20:T20,
        t21:T21):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditParameter(t16)
      auditParameter(t17)
      auditParameter(t18)
      auditParameter(t19)
      auditParameter(t20)
      auditParameter(t21)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15, t16, t17, t18, t19, t20, t21))
    }

    // 22 parameters
    def auditCall[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, T20, T21, T22, R]
    (f:Function22[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11,
      T12, T13, T14, T15, T16, T17, T18, T19, T20, T21, T22, R])
    (t1:T1, t2:T2, t3:T3, t4:T4, t5:T5, t6:T6, t7:T7, t8:T8, t9:T9, t10:T10, t11:T11,
        t12:T12, t13:T13, t14:T14, t15:T15, t16:T16, t17:T17, t18:T18, t19:T19, t20:T20,
        t21:T21, t22:T22):R = {
      auditParameter(t1)
      auditParameter(t2)
      auditParameter(t3)
      auditParameter(t4)
      auditParameter(t5)
      auditParameter(t6)
      auditParameter(t7)
      auditParameter(t8)
      auditParameter(t9)
      auditParameter(t10)
      auditParameter(t11)
      auditParameter(t12)
      auditParameter(t13)
      auditParameter(t14)
      auditParameter(t15)
      auditParameter(t16)
      auditParameter(t17)
      auditParameter(t18)
      auditParameter(t19)
      auditParameter(t20)
      auditParameter(t21)
      auditParameter(t22)
      auditReturnValue(f(t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
        t12, t13, t14, t15, t16, t17, t18, t19, t20, t21, t22))
    }

    // implicit conversions
    implicit def stringToError(s:String):Error = Error(Some(s))
    implicit def stringToWarning(s:String):Warning = Warning(Some(s))
    implicit def exceptionToError(ex:Exception):Error =
      Error(Some(ex.toString), None, Some(buildStackTraceString(ex)))
    implicit def exceptionToWarning(ex:Exception):Warning =
      Warning(Some(ex.toString), None, Some(buildStackTraceString(ex)))

    //implicit def stringToValue(s:String) = Value(s, Some("text/plain"))
    //implicit def intToValue(i:Int) = Value(i.toString, Some("text/plain"))

    def buildStackTraceString(t:Throwable):String = {
      val swriter = new StringWriter()
      val writer = new PrintWriter(swriter)
      t.printStackTrace(writer)
      writer.flush
      return swriter.toString
    }

    private lazy val localhost:String = getLocalHost().getHostName()

    def defaultOriginHostname() = localhost
  }
}
